window.Gleam.initSearch([{"doc":"eval","title":"eval","content":"# gleam_eval\n\n&gt; Chain together stateful computations in Gleam, and defer their evaluation until\nyou need them.\n\n[![Package Version](https://img.shields.io/hexpm/v/eval)](https://hex.pm/packages/eval)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/eval/)\n\n&gt; ❗️ **While the documentation for this package is still being worked on, the API\nis very much in flux. Feel free to experiment but know that things may change\nin the future!**\n\nGleam is an immutable functional programming language, but sometimes it would be\nreally helpful to model some mutable state that can be threaded through multiple\ncomputations. That&#39;s where `Eval` comes in!\n\nEverything in this package is built around the idea of an `Eval` type, which can\nbe summarised as:\n\n```gleam\nfn (context) -&gt; #(context, Result(a, e))\n```\n\nThat is, a function that takes some `context` and returns a (potentially updated)\n`context` and a `Result` of some `a` or some error `e`. With this, we can model\nall manner of computations that can be expressed in a purely functional way: we\ncan update the context, throw an error, return a value, or some combination!\n\n&gt; ❗️ This package is written in _pure Gleam_ so you can use it whether you&#39;re\ntargetting Erlang _or_ JavaScript.\n\n---\n\n## Installation\n\nIf available on Hex, this package can be added to your Gleam project:\n\n```sh\ngleam add eval\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/eval&gt;.\n\n---\n\n## Usage\n\nWe mentioned above that an `Eval` is essentially a function that takes some\n`context` and returns a `Result`. We can start to express more complex and powerful\ncomputations by composing these functions together using the various functions\nprovided in this package.\n\nTo demonstrate how everything can neatly fit together, let&#39;s write a parser for\na simple expression language. To keep things manageable, our expression language\nwill only allow for two types of expressions:\n\n- `Number`s which are just integers...\n- ...and `Add`s, which are binary expressions that add two numbers together\n\n```gleam\ntype Expr {\n  Number(Int)\n  Add(Expr, Expr)\n}\n```\n\nOur `Parser` type will be a specialisation of the `Eval` type from this package.\nThe context will be a `List(String)` that represents the current stream of\ngraphemes (Gleam doesn&#39;t have a traditional `Char` type). Additionally, we will\ndefine an `Error` type that we can throw if something goes wrong during parsing:\n\n```gleam\ntype Error {\n  Unexpected(String)\n  Eof\n  InvalidParser\n}\n```\n\nFinally, while we intend on ultimately parsing some `Expr` value, we will leave\nthe the type free in our `Parser` type, so that we can write intermediary parsers\nthat may return some other type instead (this will be useful, as we&#39;ll se in a\nmoment).\n\n```gleam\ntype Parser(a) =\n  Eval(a, Error, List(String))\n```\n\n### Basic building blocks\n\nNow we know what types we&#39;ll be dealing with, it&#39;s time to write out first parsers\nusing this package! We&#39;re going to begin by defining a `peek` and a `pop` parser\nthat will return the next grapheme in the context (and remove it, in the case of\n`pop`).\n\nFirst, make sure we&#39;ve imported everything we need:\n\n```gleam\nimport eval\nimport eval/context\nimport gleam/int\nimport gleam/list\nimport gleam/string\n```\n\nThen we can define our `peek` parser:\n\n```gleam\npub fn peek () -&gt; Parser(String) {\n  use stream &lt;- eval.try(context.get())\n\n  case stream {\n    [grapheme, ..] -&gt; eval.return(grapheme)\n    _ -&gt; eval.throw(Eof)\n  }\n}\n```\n\nThe API of this package has been designed so that (hopefully) this reads quite\nnaturally, even if you are not familiar with how things are implemented, or indeed\nGleam or functional programming in general. First we `get` the context, `then` we\nlook at the stream. If there are still graphemes left we `succeed` with the first\nelement in the stream, otherwise we `throw` with an `EOF` error.\n\nOur `pop` parser is almost identical, with the additional step of modifying the\ncontext to remove the grapheme we want to return:\n\n```gleam\npub fn pop () -&gt; Parser(String) {\n  use stream &lt;- eval.try(context.get())\n\n  case stream {\n    [grapheme, ..rest] -&gt; {\n      use _ &lt;- eval.try(context.set(rest))\n\n      eval.return(grapheme)\n    }\n    _ -&gt; eval.throw(Eof)\n  }\n}\n```\n\n### Some parser combinators\n\nWith just `peek` and `pop` we can start to build up more powerful parsers. To\nparse an integer we&#39;ll need to continuously `pop` graphemes off the stack until\nwe come across a non-digit character. We can do this by writing a `many`\ncombinator:\n\n```gleam\npub fn many (parser: Parser(a)) -&gt; Parser(List(a)) {\n  let one = fn (xs) { eval.map(parser, list.prepend(xs, _)) }\n  let go = fn (xs) {\n    eval.attempt(one, catch: fn (_) {\n      list.reverse(xs) |&gt; eval.succeed\n    })\n  }\n\n  go([])\n}\n```\n\nThis will repeatedly call `parser` until it throws, and then return the list of\nall the values it has parsed so far. The list is reversed before returning\nbecause we want to return the values in the order they were parsed.\n\n&gt; ❓ How might we modify this to parse `one_or_more` of something instead? There&#39;s\nmore than one way, but thinking about it will hopefully help you see all the\ninteresting ways `Eval` computations can be built up!\n\nIt&#39;s worth noting that `many` itself doesn&#39;t alter the context in any way. Despite\nthis, changes to the context made by running `parser` will be propagated to later\nruns as you&#39;d expect. No manual threading of state required.\n\nBefore we can parse integers, we&#39;ll want a `digit` parser to pass to `many`:\n\n```gleam\npub fn digit () -&gt; Parser(String) {\n  use grapheme &lt;- eval.try(peek())\n\n  case grapheme {\n    &quot;0&quot; | &quot;1&quot; | &quot;2&quot; | &quot;3&quot; | &quot;4&quot; |\n    &quot;5&quot; | &quot;6&quot; | &quot;7&quot; | &quot;8&quot; | &quot;9&quot; -&gt; eval.replace(pop(), grapheme)\n    _ -&gt; eval.throw(Unexpected(grapheme))\n  }\n}\n```\n\nThen, we can write an `int` parser that consumes as many digits as possible,\njoins the string back together and then uses Gleam&#39;s own `int.parse` function to\nturn that string into a bona fide integer:\n\n```gleam\npub fn int () -&gt; Parser(Int) {\n  use digits &lt;- eval.try(many(digit()))\n  let number = string.join(digits, &quot;&quot;)\n  let assert Ok(n) = int.parse(number)\n\n  eval.return(n)\n}\n```\n\nWith that out of the way, the final combinator we need is one that let&#39;s attempt\nmultiple parsers, and keep the result of whichever succeeds first. For that we\nwill define `one_of`:\n\n```gleam\npub fn one_of (parsers: List(Parser(a))) -&gt; Parser(a) {\n  case parsers {\n    [parser, ..rest] -&gt; eval.attempt(parser, catch: fn (_) { one_of(rest) })\n    _ -&gt; eval.throw(InvalidParser)\n  }\n}\n```\n\n### Parsing expressions\n\nNow we are ready to write a parser for each our of expression variants, and then\ncombine everything into a single `expr` parser. We&#39;ve seen enough of the `eval`\npackage now to be able to put together these parsers will little comment, so here\nis the rest of the program:\n\n```gleam\npub fn number () -&gt; Parser(Expr) {\n  int() |&gt; eval.map(Number)\n}\n\npub fn expr () -&gt; Parser(Expr) {\n  use lhs &lt;- eval.try(number())\n  use grapheme &lt;- eval.try(peek())\n\n  case grapheme {\n    &quot;+&quot; -&gt; {\n      use _ &lt;- eval.try(pop())\n      use rhs &lt;- eval.try(expr())\n\n      eval.return(Add(lhs, rhs))\n    }\n    _ -&gt; eval.succeed(lhs)\n  }\n}\n\npub fn run (input: String) -&gt; Result(Expr, Error) {\n  eval.run(expr(), with: string.graphemes(input))\n}\n```\n\nTake a look in `test/examples/` to see the complete parser example, as well as\na handful of others.\n","url":"index.html"},{"doc":"eval","title":"Eval","content":"pub opaque type Eval(a, e, ctx)\n A `Eval` represents a computation to be run given some context. That &quot;to be\n run&quot; part turns out to be quite powerful. By combining `Eval`s using some\n of the functions in this module, we can build up a computation that has\n access to a sort of mutable state that is updated as the computations are\n run.\n\n There are three type parameters here, not just two, because an `Eval` also\n represents a computation that can fail. In many ways, an `Eval` is just a\n superpowered `Result`!\n\n\nSynonyms:\neval.Eval\neval Eval","url":"eval.html#Eval"},{"doc":"eval","title":"all","content":"pub fn all(evals: List(Eval(a, b, c))) -&gt; Eval(List(a), b, c)\n Run a list of `Eval`s in sequence and then combine their results into a list.\n If any of the `Eval`s fail, the whole sequence fails.\n\n 📝 Note: you might find this called `sequence` in some other languages like\n Haskell or PureScript.\n\n\nSynonyms:\neval.all\neval all","url":"eval.html#all"},{"doc":"eval","title":"attempt","content":"pub fn attempt(\n  eval: Eval(a, b, c),\n  catch f: fn(c, b) -&gt; Eval(a, b, c),\n) -&gt; Eval(a, b, c)\n Run an `Eval` and then attempt to recover from an error by applying a function\n that takes the error value and returns another `Eval`.\n\nSynonyms:\neval.attempt\neval attempt","url":"eval.html#attempt"},{"doc":"eval","title":"from","content":"pub fn from(eval: fn(a) -&gt; #(a, Result(b, c))) -&gt; Eval(b, c, a)\n Construct an `Eval` from a function that takes some context and returns a pair\n of a new context and some `Result` value. This is provided as a fallback if\n none of the functions here or in `eval/context` are getting you where you need\n to go: generally you should avoid using this in favour of _combining_ the\n other functions in this module!\n\nSynonyms:\neval.from\neval from","url":"eval.html#from"},{"doc":"eval","title":"from_option","content":"pub fn from_option(value: Option(a), error: b) -&gt; Eval(a, b, c)\n Construct an `Eval` from an optional value and an error to throw if that value\n is `None`. This is useful for situations where you have some function or value\n that returns an `Option` but is not dependent on the context.\n\nSynonyms:\neval.from_option\neval from_option","url":"eval.html#from_option"},{"doc":"eval","title":"from_result","content":"pub fn from_result(value: Result(a, b)) -&gt; Eval(a, b, c)\n Construct an `Eval` from a result. This is useful for situations where you have\n some function or value that returns a `Result` but is not dependent on the\n context.\n\nSynonyms:\neval.from_result\neval from_result","url":"eval.html#from_result"},{"doc":"eval","title":"guard","content":"pub fn guard(\n  when requirement: Bool,\n  return consequence: a,\n  otherwise do: fn() -&gt; Eval(b, a, c),\n) -&gt; Eval(b, a, c)\n\n\nSynonyms:\neval.guard\neval guard","url":"eval.html#guard"},{"doc":"eval","title":"map","content":"pub fn map(\n  eval: Eval(a, b, c),\n  by f: fn(a) -&gt; d,\n) -&gt; Eval(d, b, c)\n Transform the value produced by an `Eval` using the given function.\n\n 📝 Note: you might find this called `fmap` or `&lt;$&gt;` in some other languages\n like Haskell or PureScript. In this context, the `Eval` type would be known\n as a _functor_.\n\nSynonyms:\neval.map\neval map","url":"eval.html#map"},{"doc":"eval","title":"map2","content":"pub fn map2(\n  eval_a: Eval(a, b, c),\n  eval_b: Eval(d, b, c),\n  by f: fn(a, d) -&gt; e,\n) -&gt; Eval(e, b, c)\n\n\n 📝 Note: you might find this called `liftA2` or `liftM2` in some other\n languages like Haskell or PureScript.\n\nSynonyms:\neval.map2\neval map2","url":"eval.html#map2"},{"doc":"eval","title":"map_error","content":"pub fn map_error(\n  eval: Eval(a, b, c),\n  by f: fn(b) -&gt; d,\n) -&gt; Eval(a, d, c)\n Just like `map` but for error-producing steps instead. Transforms the error\n produced by some `Eval` step using the given function.\n\nSynonyms:\neval.map_error\neval map_error","url":"eval.html#map_error"},{"doc":"eval","title":"replace","content":"pub fn replace(\n  eval: Eval(a, b, c),\n  with replacement: d,\n) -&gt; Eval(d, b, c)\n Run an `Eval` step but then replace its result with some other fixed value.\n Often used in tandem with effectful steps that often _do_ something but don&#39;t\n produce any meaninful value (and so are usually `Eval(Nil, e, ctx)`).\n\nSynonyms:\neval.replace\neval replace","url":"eval.html#replace"},{"doc":"eval","title":"replace_error","content":"pub fn replace_error(\n  eval: Eval(a, b, c),\n  with replacement: d,\n) -&gt; Eval(a, d, c)\n Just like `replace` but for error-producing steps instead. Replaces the error\n thrown by some `Eval` step with another, fixed, value.\n\nSynonyms:\neval.replace_error\neval replace_error","url":"eval.html#replace_error"},{"doc":"eval","title":"return","content":"pub fn return(value: a) -&gt; Eval(a, b, c)\n Construct an `Eval` that always succeeds with the given value, regardless of\n context.\n\n 📝 Note: you might find this called `pure` or `return` in some other languages\n like Haskell or PureScript.\n\nSynonyms:\neval.return\neval return","url":"eval.html#return"},{"doc":"eval","title":"run","content":"pub fn run(eval: Eval(a, b, c), with context: c) -&gt; Result(a, b)\n Given an `Eval`, actuall perform the computation by also providing the context\n that the computation is running in.\n\nSynonyms:\neval.run\neval run","url":"eval.html#run"},{"doc":"eval","title":"step","content":"pub fn step(eval: Eval(a, b, c), ctx: c) -&gt; #(c, Result(a, b))\n Step through an `Eval` and get back both it&#39;s result and the context it\n produced. This is especially useful if you want to run some computation,\n do some other Gleam bits, and then continue with the computation by passing\n the produced context to `run` or `step` again.\n\nSynonyms:\neval.step\neval step","url":"eval.html#step"},{"doc":"eval","title":"then","content":"pub fn then(\n  eval: Eval(a, b, c),\n  do f: fn(a) -&gt; Eval(d, b, c),\n) -&gt; Eval(d, b, c)\n Run an `Eval` and then apply a function that returns another `Eval` to the\n result. This can be useful for chaining together multiple `Eval`s.\n\n 📝 Note: you might find this called `bind`, `&gt;&gt;=`, `flatMap`, or `andThen` in\n some other languages like Haskell, Elm, or PureScript. In this context, the\n `Eval` type would be known as a _monad_.\n\nSynonyms:\neval.then\neval then","url":"eval.html#then"},{"doc":"eval","title":"throw","content":"pub fn throw(error: a) -&gt; Eval(b, a, c)\n Construct an `Eval` that always fails with the given error, regardless of\n context. Often used in combination with `then` to run some `Eval` and then\n potentially fail based on the result of that computation.\n\n \n eval(expr) |&gt; then(fn (y) {\n   case y == 0.0 {\n     True -&gt;\n       throw(DivisionByZero)\n\n     False -&gt;\n       succeed(y)\n   }\n })\n \n\nSynonyms:\neval.throw\neval throw","url":"eval.html#throw"},{"doc":"eval","title":"try","content":"pub fn try(\n  eval: Eval(a, b, c),\n  then f: fn(a) -&gt; Eval(d, b, c),\n) -&gt; Eval(d, b, c)\n Run an `Eval` and then apply a function that returns another `Eval` to the\n result. This can be useful for chaining together multiple `Eval`s. This is\n the same as [`then`](#then) but you might find the `try` naming nicer to use\n with Gleam&#39;s `use` notation.\n\n 📝 Note: you might find this called `bind`, `&gt;&gt;=`, `flatMap`, or `andThen` in\n some other languages like Haskell, Elm, or PureScript. In this context, the\n `Eval` type would be known as a _monad_.\n\nSynonyms:\neval.try\neval try","url":"eval.html#try"},{"doc":"eval","title":"eval","content":"","url":"eval.html"},{"doc":"eval/context","title":"get","content":"pub fn get() -&gt; Eval(a, b, a)\n Get the current context.\n\nSynonyms:\neval/context.get\neval/context get","url":"eval/context.html#get"},{"doc":"eval/context","title":"set","content":"pub fn set(ctx: a) -&gt; Eval(Nil, b, a)\n Replace the current context with a new fixed value.\n\nSynonyms:\neval/context.set\neval/context set","url":"eval/context.html#set"},{"doc":"eval/context","title":"update","content":"pub fn update(f: fn(a) -&gt; a) -&gt; Eval(Nil, b, a)\n Update the current context by applying a function to it.\n\nSynonyms:\neval/context.update\neval/context update","url":"eval/context.html#update"},{"doc":"eval/context","title":"eval/context","content":"","url":"eval/context.html"}]);